@@ -1,6 +1,9 @@
+context("test-vignettes")
+
 test_that("Test non-negative least squares", {
     require(MASS)
     print("LS")
+
     ## Generate problem data
     s <- 1
     m <- 10
@@ -55,9 +58,10 @@ test_that("Test censored regression", {
   n <- 30
   M <- 50
   K <- 200
+  set.seed(n*M*K)
 
   print("CENSORED")
-  set.seed(n*M*K)
+
   X <- matrix(stats::rnorm(K*n), nrow = K, ncol = n)
   beta_true <- matrix(stats::rnorm(n), nrow = n, ncol = 1)
   y <- X %*% beta_true + 0.3*sqrt(n)*stats::rnorm(K)
@@ -84,6 +88,7 @@ test_that("Test censored regression", {
   obj <- sum((y_censored - X_ordered %*% beta)^2)
   prob <- Problem(Minimize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_ols <- result$getValue(beta)
   plot_results(beta_ols)
 
@@ -91,6 +96,7 @@ test_that("Test censored regression", {
   obj <- sum((y_censored[1:M] - X_ordered[1:M,] %*% beta)^2)
   prob <- Problem(Minimize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_unc <- result$getValue(beta)
   plot_results(beta_unc)
 
@@ -99,13 +105,15 @@ test_that("Test censored regression", {
   constr <- list(X_ordered[(M+1):K,] %*% beta >= D)
   prob <- Problem(Minimize(obj), constr)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_cens <- result$getValue(beta)
   plot_results(beta_cens)
 })
 
 test_that("Test Elastic-net regression", {
   set.seed(1)
-    print("ELASTIC")
+  print("ELASTIC")
+
   # Problem data
   n = 20
   m = 1000
@@ -139,6 +147,7 @@ test_that("Test Elastic-net regression", {
     obj <- loss + elastic_reg(beta, lambda, alpha)
     prob <- Problem(Minimize(obj))
     result <- solve(prob)
+    expect_equal(result$status, "optimal")
     beta_vals[,i] <- result$getValue(beta)
   }
 
@@ -160,6 +169,7 @@ test_that("Test Huber regression", {
   p <- 0.1    ## Fraction of responses with sign flipped
 
   print("HUBER")
+
   ## Generate problem data
   beta_true <- 5*matrix(stats::rnorm(n), nrow = n)
   X <- matrix(stats::rnorm(m*n), nrow = m, ncol = n)
@@ -177,6 +187,7 @@ test_that("Test Huber regression", {
   obj <- sum((y - X %*% beta)^2)
   prob <- Problem(Minimize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_ols <- result$getValue(beta)
   err_ols <- result$getValue(rel_err)
 
@@ -189,6 +200,7 @@ test_that("Test Huber regression", {
   obj <- sum(CVXR::huber(y - X %*% beta, M))
   prob <- Problem(Minimize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_hub <- result$getValue(beta)
   err_hub <- result$getValue(rel_err)
   lines(X, X %*% beta_hub, col = "seagreen", lty = "dashed")
@@ -197,6 +209,7 @@ test_that("Test Huber regression", {
   obj <- sum((y - factor*(X %*% beta))^2)
   prob <- Problem(Minimize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   beta_prs <- result$getValue(beta)
   err_prs <- result$getValue(rel_err)
   lines(X, X %*% beta_prs, col = "black")
@@ -209,7 +222,9 @@ test_that("Test logistic regression", {
   offset <- 0
   sigma <- 45
   DENSITY <- 0.2
+
   print("LOGISTIC")
+
   beta_true <- stats::rnorm(n)
   idxs <- sample(n, size = floor((1-DENSITY)*n), replace = FALSE)
   beta_true[idxs] <- 0
@@ -220,6 +235,7 @@ test_that("Test logistic regression", {
   obj <- -sum(logistic(-X[y <= 0,] %*% beta)) - sum(logistic(X[y == 1,] %*% beta))
   prob <- Problem(Maximize(obj))
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
 
   log_odds <- result$getValue(X %*% beta)
   beta_res <- result$getValue(beta)
@@ -230,7 +246,8 @@ test_that("Test logistic regression", {
 
 test_that("Test saturating hinges problem", {
   require(ElemStatLearn)
-    print("SAT HINGES")
+  print("SAT HINGES")
+
   ## Import and sort data
   data(bone)
   X <- bone[bone$gender == "female",]$age
@@ -271,6 +288,7 @@ test_that("Test saturating hinges problem", {
 
     ## Solve problem and save spline weights
     result <- solve(prob)
+    expect_equal(result$status, "optimal")
     w0s <- result$getValue(w0)
     ws <- result$getValue(w)
     splines[,i] <- f_est(xrange, knots, w0s, ws)
@@ -294,6 +312,7 @@ test_that("Test saturating hinges problem", {
   loss <- sum(loss_obs(y_all, f_est(X_all, knots, w0, w)))
   prob <- Problem(Minimize(loss + reg), constr)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   spline_sq <- f_est(xrange, knots, result$getValue(w0), result$getValue(w))
 
   ## Solve with Huber loss
@@ -301,6 +320,7 @@ test_that("Test saturating hinges problem", {
   loss <- sum(loss_obs(y, f_est(X, knots, w0, w), 0.01))
   prob <- Problem(Minimize(loss + reg), constr)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   spline_hub <- f_est(xrange, knots, result$getValue(w0), result$getValue(w))
 
   ## Compare fitted functions with squared error and Huber loss
@@ -312,7 +332,8 @@ test_that("Test saturating hinges problem", {
 
 test_that("Test log-concave distribution estimation", {
   set.seed(1)
-    print("LOG CONCAVE")
+  print("LOG CONCAVE")
+
   ## Calculate a piecewise linear function
   pwl_fun <- function(x, knots) {
     n <- nrow(knots)
@@ -350,6 +371,7 @@ test_that("Test log-concave distribution estimation", {
   constraints <- list(sum(exp(u)) <= 1, diff(u[1:K]) >= diff(u[2:(K+1)]))
   prob <- Problem(Maximize(obj), constraints)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   pmf <- result$getValue(exp(u))
 
   ## Plot probability mass function
@@ -375,17 +397,21 @@ test_that("Test channel capacity problem", {
   m <- 2
   P <- rbind(c(0.75, 0.25),    ## Channel transition matrix
              c(0.25, 0.75))
+
   print("CHANNEL")
+
   ## Form problem
   x <- Variable(n)   ## Probability distribution of input signal x(t)
   y <- P %*% x       ## Probability distribution of output signal y(t)
   c <- apply(P * log2(P), 2, sum)
-  I <- c %*% x + sum(entr(y))   ## Mutual information between x and y
+  I <- t(c) %*% x + sum(entr(y))   ## Mutual information between x and y
   obj <- Maximize(I)
   constraints <- list(sum(x) == 1, x >= 0)
   prob <- Problem(obj, constraints)
-
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
+
+  ## Optimal channel capacity and input signal.
   result$value
   result$getValue(x)
 })
@@ -397,6 +423,7 @@ test_that("Test optimal allocation in a Gaussian broadcast channel", {
   beta <- seq(10, n-1+10)/n
   P_tot <- 0.5
   W_tot <- 1.0
+
   print("OPTIMAL ALLOCATION")
 
   ## Form problem
@@ -407,6 +434,7 @@ test_that("Test optimal allocation in a Gaussian broadcast channel", {
   constraints <- list(P >= 0, W >= 0, sum(P) == P_tot, sum(W) == W_tot)
   prob <- Problem(objective, constraints)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
 
   ## Optimal utility, power, and bandwidth
   -result$value
@@ -432,6 +460,7 @@ test_that("Test catenary problem", {
   ## Solve the catenary problem
   prob <- Problem(objective, constraints)
   system.time(result <- solve(prob))
+  expect_equal(result$status, "optimal")
 
   ## Plot and compare with ideal catenary
   xs <- result$getValue(x)
@@ -463,6 +492,7 @@ test_that("Test catenary problem", {
   constraints[[4]] <- (y[m] == 0.5)
   prob <- Problem(objective, constraints)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
 
   ## Plot catenary against ground
   xs <- result$getValue(x)
@@ -475,10 +505,11 @@ test_that("Test catenary problem", {
 })
 
 test_that("Test direct standardization problem", {
-    print("DIRECT")
-    skew_sample <- function(data, bias) {
-    if(missing(bias))
-      bias <- rep(1.0, ncol(data))
+  print("DIRECT")
+
+  skew_sample <- function(data, bias) {
+  if(missing(bias))
+    bias <- rep(1.0, ncol(data))
     num <- exp(data %*% bias)
     return(num / sum(num))
   }
@@ -517,6 +548,7 @@ test_that("Test direct standardization problem", {
 
   ## Solve for the distribution weights
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   weights <- result$getValue(w)
 
   ## Plot probability density function
@@ -535,8 +567,9 @@ test_that("Test direct standardization problem", {
 })
 
 test_that("Test risk-return tradeoff in portfolio optimization", {
-    print("PORTFOLIO")
-    ## Problem data
+  print("PORTFOLIO")
+
+  ## Problem data
   set.seed(10)
   n <- 10
   SAMPLES <- 100
@@ -562,6 +595,7 @@ test_that("Test risk-return tradeoff in portfolio optimization", {
     objective <- ret - gamma * risk
     prob <- Problem(Maximize(objective), constraints)
     result <- solve(prob)
+    expect_equal(result$status, "optimal")
 
     ## Evaluate risk/return for current solution
     risk_data[i] <- result$getValue(sqrt(risk))
@@ -589,8 +623,9 @@ test_that("Test risk-return tradeoff in portfolio optimization", {
 })
 
 test_that("Test Kelly gambling optimal bets", {
-    print("KELLY")
-    set.seed(1)
+  print("KELLY")
+
+  set.seed(1)
   n <- 20      ## Total bets
   K <- 100     ## Number of possible returns
   PERIODS <- 100
@@ -614,6 +649,7 @@ test_that("Test Kelly gambling optimal bets", {
   constraints <- list(sum(b) == 1, b >= 0)
   prob <- Problem(obj, constraints)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   bets <- result$getValue(b)
 
   ## Naive betting scheme: bet in proportion to expected return
@@ -644,8 +680,9 @@ test_that("Test Kelly gambling optimal bets", {
 })
 
 test_that("Test worst-case covariance", {
-    print("WORST CASE")
-    ## Problem data
+  print("WORST CASE")
+
+  ## Problem data
   w <- matrix(c(0.1, 0.2, -0.05, 0.1))
   ## Constraint matrix:
   ## [[0.2, + ,  +,   +-],
@@ -654,21 +691,30 @@ test_that("Test worst-case covariance", {
   ##  [+-,  -,   +,  0.1]]
 
   ## Form problem
-  Sigma <- Semidef(4)
+  Sigma <- Variable(4, 4, PSD = TRUE)
   obj <- Maximize(t(w) %*% Sigma %*% w)
   constraints <- list(Sigma[1,1] == 0.2, Sigma[2,2] == 0.1, Sigma[3,3] == 0.3, Sigma[4,4] == 0.1,
                       Sigma[1,2] >= 0, Sigma[1,3] >= 0, Sigma[2,3] <= 0, Sigma[2,4] <= 0, Sigma[3,4] >= 0)
   prob <- Problem(obj, constraints)
   result <- solve(prob, solver = "SCS")
+  expect_equal(result$status, "optimal")
   print(result$getValue(Sigma))
 
-  Sigma_true <- rbind(c(0.2,    0.0973, 0,     0.0750),
-                      c(0.0973, 0.1,   -0.1020, 0),
-                      c(0,     -0.1020,  0.3,   0.0016),
-                      c(0.0750, 0,      0.0016,     0.1))
-  dimnames(Sigma_true) <- list(NULL, NULL)
+  ## Sigma_true <- rbind(c(0.2,    0.0978, 0,     0.0741),
+  ##                     c(0.0978, 0.1,   -0.101, 0),
+  ##                     c(0,     -0.101,  0.3,   0),
+  ##                     c(0.0741, 0,      0,     0.1))
+
+  ## The new SCS solver gives slightly different results from the above
+  ## So the values below have been obtained from cvxpy 1.25.
+
+  Sigma_true <- rbind(c(0.2,    0.09674, 0,     0.0762),
+                      c(0.09674, 0.1,   -0.103, 0),
+                      c(0,     -0.103,  0.3,   0.0041),
+                      c(0.0762, 0,      .0041,     0.1))
+
   expect_equal(sqrt(result$value), 0.1232, tolerance = 1e-3)
-  expect_equal(as.matrix(result$getValue(Sigma)), Sigma_true, tolerance = 1e-3)
+  expect_equal(result$getValue(Sigma), Sigma_true, tolerance = 1e-3)
 
   ## Problem data
   n <- 5
@@ -684,18 +730,20 @@ test_that("Test worst-case covariance", {
   U <- Sigma_nom + Delta
   L <- Sigma_nom - Delta
 
-  Sigma <- Semidef(n)
+  Sigma <- Variable(n, n, PSD = TRUE)
   obj <- quad_form(w, Sigma)
   constr <- list(L <= Sigma, Sigma <= U, Sigma == t(Sigma))
   prob <- Problem(Maximize(obj), constr)
   result <- solve(prob)
+  expect_equal(result$status, "optimal")
   print(result$getValue(Sigma))
 })
 
 test_that("Test sparse inverse covariance estimation", {
   require(Matrix)
   require(expm)
-    print("SPARSE INV")
+  print("SPARSE INV")
+
   set.seed(1)
   n <- 10      ## Dimension of matrix
   m <- 1000    ## Number of samples
@@ -714,7 +762,7 @@ test_that("Test sparse inverse covariance estimation", {
   x_sample <- matrix(stats::rnorm(n*m), nrow = m, ncol = n) %*% t(expm::sqrtm(R))
   Q <- cov(x_sample)    ## Sample covariance matrix
 
-  S <- Semidef(n)    ## Variable constrained to positive semidefinite cone
+  S <- Variable(n, n, PSD = TRUE) ## Variable constrained to positive semidefinite cone
   obj <- Maximize(log_det(S) - matrix_trace(S %*% Q))
   for(alpha in alphas) {
     constraints <- list(sum(abs(S)) <= alpha)
@@ -722,6 +770,7 @@ test_that("Test sparse inverse covariance estimation", {
     ## Form and solve optimization problem
     prob <- Problem(obj, constraints)
     result <- solve(prob)
+    expect_equal(result$status, "optimal")
 
     ## Create covariance matrix
     R_hat <- base::solve(result$getValue(S))
@@ -733,8 +782,9 @@ test_that("Test sparse inverse covariance estimation", {
 })
 
 test_that("Test fastest mixing Markov chain (FMMC)", {
-    print("MCMC")
-    ## Boyd, Diaconis, and Xiao. SIAM Rev. 46 (2004) pgs. 667-689 at pg. 672
+  print("MCMC")
+
+  ## Boyd, Diaconis, and Xiao. SIAM Rev. 46 (2004) pgs. 667-689 at pg. 672
   ## Form the complementary graph
   antiadjacency <- function(g) {
     n <- max(as.numeric(names(g)))   ## Assumes names are integers starting from 1
@@ -766,6 +816,7 @@ test_that("Test fastest mixing Markov chain (FMMC)", {
     }
     prob <- Problem(objective, constraints)
     result <- solve(prob)
+    expect_equal(result$status, "optimal")
     if(verbose)
       cat("Status: ", result$status, ", Optimal Value = ", result$value)
     list(status = result$status, value = result$value, P = result$getValue(P))
