@@ -1,3 +1,4 @@
+context("test-g01-grad")
 a <- Variable(name = "a")
 
 x <- Variable(2, name = "x")
@@ -10,7 +11,7 @@ C <- Variable(3, 2, name = "C")
 test_that("Test gradient for affine_prod", {
   value(C) <- rbind(c(1,-2), c(3,4), c(-1,-3))
   value(A) <- rbind(c(3,2), c(-5,1))
-  expr <- affine_prod(C, A)
+  expect_warning(expr <- C %*% A)
 
   expect_equivalent(as.matrix(grad(expr)[[as.character(C@id)]]), rbind(c(3,0,0,2,0,0), c(0,3,0,0,2,0), c(0,0,3,0,0,2),
                                                     c(-5,0,0,1,0,0), c(0,-5,0,0,1,0), c(0,0,-5,0,0,1)))
@@ -247,13 +248,13 @@ test_that("Test linearize method", {
   value(A) <- cbind(c(1,2), c(3,4))
   expr <- A^2 + 5
   lin_expr <- linearize(expr)
-  manual <- value(expr) + 2*reshape_expr(value(diag(vec(A))) %*% vec(A - value(A)), 2, 2)
+  manual <- value(expr) + 2*reshape_expr(value(diag(vec(A))) %*% vec(A - value(A)), c(2, 2))
   expect_equivalent(as.matrix(value(lin_expr)), value(expr))
 
   value(A) <- rbind(c(-5,-5), c(8.2,4.4))
   expr <- A^2 + 5
   lin_expr <- linearize(expr)
-  manual <- value(expr) + 2*reshape_expr(value(diag(vec(A))) %*% vec(A - value(A)), 2, 2)
+  manual <- value(expr) + 2*reshape_expr(value(diag(vec(A))) %*% vec(A - value(A)), c(2, 2))
   expect_true(all(value(lin_expr) <= value(expr)))
   expect_equivalent(as.matrix(value(lin_expr)), value(manual))
 
@@ -283,8 +284,7 @@ test_that("Test gradient for log", {
 
   value(a) <- -1
   expr <- log(a)
-  expect_warning(expr_grad <- grad(expr)[[as.character(a@id)]])
-  expect_true(is.na(expr_grad))
+  expect_true(is.na(grad(expr)[[as.character(a@id)]]))
 
   value(x) <- c(3,4)
   expr <- log(x)
@@ -293,8 +293,7 @@ test_that("Test gradient for log", {
 
   value(x) <- c(-1e-9,4)
   expr <- log(x)
-  expect_warning(expr_grad <- grad(expr)[[as.character(x@id)]])
-  expect_true(is.na(expr_grad))
+  expect_true(is.na(grad(expr)[[as.character(x@id)]]))
 
   value(A) <- cbind(c(1,2), c(3,4))
   expr <- log(A)
@@ -618,7 +617,7 @@ test_that("Test domain for power", {
   expect_equivalent(as.matrix(grad(expr)[[as.character(x@id)]]), matrix(0, nrow = 2, ncol = 2))
 })
 
-test_that("Test grad for partial minimization/maximization problems", {
+# test_that("Test grad for partial minimization/maximization problems", {
   # for(obj in list(Minimize(a^-1), Maximize(Entr(a)))) {
   #   prob <- Problem(obj, list(x + a >= c(5,8)))
   #
@@ -663,7 +662,7 @@ test_that("Test grad for partial minimization/maximization problems", {
   #   grad <- grad(expr)
   #   expect_equal(grad, list())
   # }
-})
+# })
 
 test_that("Test grad for affine atoms", {
   value(a) <- 2
